/*
 *	Videot Decoder
 *
 *	The basic design is straight out of Mike Singletons 
 *	"Doomdarks Revenge" from back in the early 1980's. Mike used this
 *	scheme to encode the 3d landscape objects to fit in a 48K spectrum
 *
 *	We add the obvious xor option for progressive changes (may not be a win)
 *	and a shorter encoding for unchanged lines. Adding encodings for
 *	slide left, slide right, and bigger/smaller is the next project
 */

#include <stdio.h> 
#include "vidiot.h"
 
static u8 get_byte(struct video *v)
{
	int c=getchar();
	if(c==EOF)
		exit(0);
	return (u8)c;
}

static void videot_decode_line(struct video *v, u8 dp)
{
	int white=0;
	int x=0;
	while(x<v->width)
	{
		if(x)
			dp = get_byte(v);
		fprintf(stderr, "%d, %d = %d (%d)\n", x, dp, x+dp, v->width);
		draw_line(v, dp, white);
		x += dp;
		white^=1;
	}
	if(x!=v->width)
	{
		fprintf(stderr, "Misformat -> %d.\n", get_byte(v));
		exit(1);
	}
}

		
static void videot_decode_xor(struct video *v)
{
	int xor_bit = 0;
	int x=0;
	while(x<v->width)
	{
		u8 dp = get_byte(v);
		fprintf(stderr, "%d, %d = %d (%d)\n", x, dp, x+dp, v->width);
		draw_xor_line(v, dp, xor_bit);
		x += dp;
		xor_bit^=1;
	}
	if(x!=v->width)
	{
		fprintf(stderr, "Misformat -> %d.\n", get_byte(v));
		exit(1);
	}
}


int videot_decode_progressive(struct video *v)
{	
	int y = 0;


	while(y<v->height)
	{
		u8 dp;

		memcpy(v->oldrow[y], v->row[y], v->width);
#ifdef SYNC_TEST
		if(get_byte(v)!=255 || get_byte(v) !=254)
		{
			fprintf(stderr, "Out of sync on line %d.\n", y);
			exit(1);
		}
#endif
		
		dp = get_byte(v);
		set_draw_row(v, y);

		printf("OP=%d, Y=%d\n", dp, y);
	
		if(dp == 0)
		{
			fprintf(stderr, "XORL\n");
			videot_decode_xor(v);
		}
		else if(dp < 4) 	/* unusual case picked for efficiency */
		{
			fprintf(stderr, "SKIPL\n", 2+2*dp);
			y+=2+2*dp;
			continue;
		}
		else
			videot_decode_line(v, dp);

#ifdef SYNC_TEST
		if(get_byte(v)!=255 || get_byte(v)!=255)
		{
			fprintf(stderr, "Out of sync end of line %d.\n", y);
			exit(1);
		}
#endif	
		y++;
	}
}

int main(int argc, char *argv[])
{
	static struct video v;
	memset(&v, 0, sizeof(v));
	v.width = 168;
	v.height = 120;

	gtk_setup(argc, argv, &v);
}

